# Copyright (C) 2002-2017 CERN for the benefit of the ATLAS collaboration

#!/usr/bin/env python
import sys
from AthenaConfiguration.ComponentAccumulator import ComponentAccumulator
from AthenaConfiguration.ComponentFactory import CompFactory
from AthenaCommon.Configurable import Configurable
from CalypsoConfiguration.AllConfigFlags import ConfigFlags
from CalypsoConfiguration.MainServicesConfig import MainServicesCfg
from AthenaPoolCnvSvc.PoolReadConfig import PoolReadCfg
from FaserSCT_GeoModel.FaserSCT_GeoModelConfig import FaserSCT_GeometryCfg
from TrackerPrepRawDataFormation.TrackerPrepRawDataFormationConfig import FaserSCT_ClusterizationCfg

readoutmode='HIT' #or HIT, LEVEL, EDGE




def TrackerEfficiencyExampleCfg(flags, **kwargs):

    acc = FaserSCT_GeometryCfg(flags)
    kwargs.setdefault("TrackCollection", "SegmentFit")#change to seg
    TrackerEfficiencyAlg = CompFactory.TrackerEfficiencyAlg
    acc.addEventAlgo(TrackerEfficiencyAlg(**kwargs))

    thistSvc = CompFactory.THistSvc()
    thistSvc.Output = [f"HIST DATAFILE='trackcount{readoutmode}.root' OPT='RECREATE'"]
    acc.addService(thistSvc)

    return acc

def RunTrackerEfficiency(mode):
  #log.setLevel(DEBUG)

    Configurable.configurableRun3Behavior = True

    # Configure
    ConfigFlags.Input.Files = [f'{mode}/Faser-Physics-003500-00000-xAOD.root']
    ConfigFlags.GeoModel.FaserVersion = "FASER-TB00"
    ConfigFlags.IOVDb.GlobalTag = "OFLCOND-FASER-TB00" 
    ConfigFlags.IOVDb.DatabaseInstance = "OFLP200"
    ConfigFlags.GeoModel.Align.Dynamic = False
    ConfigFlags.lock()

    # Core components
    acc = MainServicesCfg(ConfigFlags)
    acc.merge(PoolReadCfg(ConfigFlags))

    # Inner Detector
    acc.merge(TrackerEfficiencyExampleCfg(mode,ConfigFlags))

    replicaSvc = acc.getService("DBReplicaSvc")
    replicaSvc.COOLSQLiteVetoPattern = ""
    replicaSvc.UseCOOLSQLite = True
    replicaSvc.UseCOOLFrontier = False
    replicaSvc.UseGeomSQLite = True

    # Execute and finish
    sys.exit(int(acc.run(maxEvents=-1).isFailure()))


if __name__ == "__main__":
    #log.setLevel(DEBUG)

    Configurable.configurableRun3Behavior = True

   # Configure
    ConfigFlags.Input.Files = [f'{readoutmode}/Faser-Physics-003500-00000-xAOD.root']
    ConfigFlags.GeoModel.FaserVersion = "FASER-TB00"
    ConfigFlags.IOVDb.GlobalTag = "OFLCOND-FASER-TB00" 
    ConfigFlags.IOVDb.DatabaseInstance = "OFLP200"
    ConfigFlags.GeoModel.Align.Dynamic = False
    ConfigFlags.lock()

    # Core components
    acc = MainServicesCfg(ConfigFlags)
    acc.merge(PoolReadCfg(ConfigFlags))

    # Inner Detector
    acc.merge(TrackerEfficiencyExampleCfg(ConfigFlags))

    replicaSvc = acc.getService("DBReplicaSvc")
    replicaSvc.COOLSQLiteVetoPattern = ""
    replicaSvc.UseCOOLSQLite = True
    replicaSvc.UseCOOLFrontier = False
    replicaSvc.UseGeomSQLite = True

    # Execute and finish
    sys.exit(int(acc.run(maxEvents=-1).isFailure()))
